package app;

import java.text.DecimalFormat;

/**
 * @author Petar Čolić Matrica se sastoji od polja doubleova dimenzija r,s.
 *         epsilon je tolerirana pogreška, matrica P je permutacijska matrica.
 *         Razred može riješiti probleme tipa Ax=b pomoću LU i LUP
 *         dekompozicije, te supstitucija unaprijed i unatrag. Metode rade
 *         kopije elemenata, tako da se ne koristi isti memorijski prostor, no
 *         referenca se može opet usmjeriti nazad.
 */
public class Matrix {

	public double[][] polje;
	public int r, s;
	double epsilon = Math.pow(10, -6);

	DecimalFormat df = new DecimalFormat("00.00");

	/**
	 * Prazan konstruktor
	 */
	public Matrix() {

	}

	/**
	 * Konstruktor za app
	 * 
	 * @param x
	 *            varijabla x
	 * @param y
	 *            varijabla y
	 */
	public Matrix(double x, double y) {
		this.r = 1;
		this.s = 2;
		this.polje = new double[r][s];
		polje[0][0] = x;
		polje[0][1] = y;

	}

	/**
	 * Konstruira matricu dimenzija RxS
	 * 
	 * @param r
	 *            broj redova
	 * @param s
	 *            broj stupaca
	 */
	public Matrix(int r, int s) {
		this.r = r;
		this.s = s;
		this.polje = new double[r][s];
	}

	/**
	 * Konstruira kvadratnu matricu dimenzija RxR
	 * 
	 * @param r
	 *            broj redova i stupaca
	 */
	public Matrix(int r) {
		this.r = this.s = r;
		this.polje = new double[r][r];
	}

	/**
	 * Metoda mijenja dimenzije matrice
	 * 
	 * @param r
	 *            novi broj redaka
	 * @param s
	 *            novi broj stupaca
	 */
	public void promjenaDimenzija(int r, int s) {
		this.polje = new double[r][s];
	}

	/**
	 * Vraća kopiju matrice - kopiraju se parametri r,s te polje doublova
	 * 
	 * @return kopija matrice
	 */
	public Matrix dajKopiju() {
		Matrix a = new Matrix(r, s);
		a.df = this.df;
		for (int i = 0; i < r; i++)
			for (int j = 0; j < s; j++)
				a.polje[i][j] = this.polje[i][j];
		return a;
	}

	/**
	 * Dohvaća traženi element matrice.
	 * 
	 * @param r
	 *            redak elementa
	 * @param s
	 *            stupac elementa
	 * @return
	 */
	public double dohvati(int r, int s) {
		return polje[r][s];
	}

	/**
	 * Postavlja određeni element matrice na neku vrijednost.
	 * 
	 * @param r
	 *            redak elementa
	 * @param s
	 *            stupac elementa
	 * @param value
	 *            nova vrijednost elementa
	 */
	public void postavi(int r, int s, double value) {
		polje[r][s] = value;
	}

	/**
	 * Zbraja dvije matrice.
	 * 
	 * @param b
	 *            matrica s kojom se zbraja
	 * @return rezultat, nova matrica
	 */
	public Matrix zbrojiSa(Matrix b) {
		if (this.r != b.r || this.s != b.s) {
			System.out.println("Ne mogu zbrojiti matrice!");
			System.exit(1);
		}
		Matrix c = new Matrix(this.r, this.s);
		for (int i = 0; i < b.r; i++)
			for (int j = 0; j < b.s; j++)
				c.polje[i][j] = this.polje[i][j] + b.polje[i][j];
		return c;
	}

	/**
	 * Oduzima dvije matrice.
	 * 
	 * @param b
	 *            matrica od koje se oduzima
	 * @return rezultat, nova matrica
	 */
	public Matrix oduzmiOd(Matrix b) {
		if (this.r != b.r || this.s != b.s) {
			System.out.println("Ne mogu oduzeti matrice!");
			System.exit(1);
		}
		Matrix c = new Matrix(this.r, this.s);
		for (int i = 0; i < b.r; i++)
			for (int j = 0; j < b.s; j++)
				c.polje[i][j] = b.polje[i][j] - this.polje[i][j];
		return c;
	}

	/**
	 * Mnozi dvije matrice.
	 * 
	 * @param b
	 *            matrica sa kojom se mnozi
	 * @return rezultat, nova matrica
	 */
	public Matrix mnoziSa(Matrix b) {
		if (this.s != b.r) {
			System.out.println("Ne mogu mnoziti matrice!");
			System.exit(1);
		}
		Matrix c = new Matrix(this.r, b.s);
		for (int i = 0; i < c.r; i++)
			for (int j = 0; j < c.s; j++)
				for (int k = 0; k < this.s; k++)
					c.polje[i][j] += (this.polje[i][k] * b.polje[k][j]);
		return c;
	}

	/**
	 * Transponira matricu.
	 * 
	 * @return nova, transponirana matrica
	 */
	public Matrix transponiraj() {
		Matrix c = new Matrix(s, r);
		for (int i = 0; i < r; i++)
			for (int j = 0; j < s; j++)
				c.polje[j][i] = polje[i][j];

		return c;
	}

	/**
	 * Mnozi matricu skalarom
	 * 
	 * @param skalar
	 *            skalar kojim se mnozi matrica
	 * @return rezultat, nova matrica
	 */
	public Matrix mnoziSaSkalarom(double skalar) {
		Matrix c = new Matrix(r, s);
		for (int i = 0; i < r; i++)
			for (int j = 0; j < s; j++)
				c.polje[i][j] = this.polje[i][j] * skalar;
		return c;
	}

	/**
	 * Za ispis u GUI
	 */
	@Override
	public String toString() {
		return String.valueOf(polje[0][0]) + ", " + String.valueOf(polje[0][1]);
	}

	/**
	 * Provjerava da li su matrice jednake - dimenzija 1x2
	 * 
	 * @param x
	 *            matrica s kojom se uspoređuje
	 * @return true ako su jednake
	 */
	public boolean isti(Matrix x) {
		if (this.polje[0][0] == x.polje[0][0]
				&& this.polje[0][1] == x.polje[0][1])
			return true;
		else
			return false;
	}

}
